@@ -1,21 +1,20 @@
 #include <config.h>
 #include <string.h>
-#include <bonobo-activation/bonobo-activation.h>
+#include <stdlib.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnome/gnome-config.h>
+#include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-client.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include <gconf/gconf-client.h>
-#include <bonobo.h>
-#include <bonobo/bonobo-ui-main.h>
 #include <eel/eel-vfs-extensions.h>
 #include "eog-hig-dialog.h"
 #include "eog-window.h"
 #include "session.h"
 #include "eog-config-keys.h"
 
-static void open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data); 
+static void open_uri_list_cb (EogWindow *window, GSList *uri_list, gpointer data); 
 static GtkWidget* create_new_window (void);
 
 
@@ -166,27 +165,32 @@ make_canonical_uri (const char *path)
  * regular files and one list with all directories.
  **/
 static void
-sort_files (GList *files, GList **file_list, GList **dir_list, GList **error_list)
+sort_files (GSList *files, GList **file_list, GList **dir_list, GList **error_list)
 {
-	GList *it;
+	GSList *it;
 	GnomeVFSFileInfo *info;
 	
 	info = gnome_vfs_file_info_new ();
 
 	for (it = files; it != NULL; it = it->next) {
 		GnomeVFSURI *uri;
-		GnomeVFSResult result;
+		GnomeVFSResult result = GNOME_VFS_OK;
 		char *filename;
 		
 		uri = make_canonical_uri ((char*)it->data);
 
-		result = gnome_vfs_get_file_info_uri (uri, info,
-						      GNOME_VFS_FILE_INFO_DEFAULT |
-						      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+		if (uri != NULL) {
+			result = gnome_vfs_get_file_info_uri (uri, info,
+							      GNOME_VFS_FILE_INFO_DEFAULT |
+							      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 
-		filename = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+			filename = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+		}
+		else {
+			filename = g_strdup ((char*) it->data);
+		}
 
-		if (result != GNOME_VFS_OK)
+		if (result != GNOME_VFS_OK || uri == NULL)
 			*error_list = g_list_append (*error_list, filename);
 		else {
 			if (info->type == GNOME_VFS_FILE_TYPE_REGULAR)
@@ -197,7 +201,9 @@ sort_files (GList *files, GList **file_list, GList **dir_list, GList **error_lis
 				*error_list = g_list_append (*error_list, filename);
 		}
 
-		gnome_vfs_uri_unref (uri);
+		if (uri != NULL) {
+			gnome_vfs_uri_unref (uri);
+		}
 		gnome_vfs_file_info_clear (info);
 	}
 
@@ -227,10 +233,10 @@ user_wants_collection (gint n_windows)
 	int ret;
 
 	g_snprintf (body, 128,
-		    _("You are about to open %i windows simultanously. Do you want to open them in a collection instead?"), 
+		    _("You are about to open %i windows simultaneously. Do you want to open them in a collection instead?"), 
 		    n_windows);
 
-	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_WARNING,
+	dlg = eog_hig_dialog_new (NULL, GTK_STOCK_DIALOG_WARNING,
 				  _("Open multiple single windows?"),
 				  body,
 				  TRUE);
@@ -240,7 +246,6 @@ user_wants_collection (gint n_windows)
 	gtk_dialog_add_button (GTK_DIALOG (dlg), _("Collection"), COLLECTION_YES);
        	gtk_dialog_set_default_response (GTK_DIALOG (dlg), COLLECTION_YES);
 
-
 	gtk_widget_show_all (dlg);
 
 	ret = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -253,91 +258,71 @@ user_wants_collection (gint n_windows)
 	return ret;
 }
 
-/* Concatenates the strings in a list and separates them with newlines.  If the
- * list is empty, returns the empty string.  Returns the number of list elements in n.
- */
-static char *
-concat_string_list_with_newlines (GList *strings, int *n)
-{
-	int len;
-	GList *l;
-	char *str, *p;
-
-	*n = 0;
-
-	if (!strings)
-		return g_strdup ("");
-
-	len = 0;
-
-	for (l = strings; l; l = l->next) {
-		char *s;
-
-		(*n)++;
-
-		s = l->data;
-		len += strlen (s) + 1; /* Add 1 for newline */
-	}
-
-	str = g_new (char, len);
-	p = str;
-	for (l = strings; l; l = l->next) {
-		char *s;
-
-		s = l->data;
-		p = g_stpcpy (p, s);
-		*p++ = '\n';
-	}
-
-	p--;
-	*p = '\0';
-
-	return str;
-}
-
 /* Shows an error dialog for files that do not exist */
 static void
 show_nonexistent_files (GList *error_list)
 {
-	char *str;
-	int n;
 	GtkWidget *dlg;
-	char *path;
+	GList *it;
+	int n = 0;
+	int len;
+	GString *detail;
 
 	g_assert (error_list != NULL);
 
-	str = concat_string_list_with_newlines (error_list, &n);
+	len = g_list_length (error_list);
+	detail = g_string_new ("");
 
-	path = gnome_vfs_format_uri_for_display (str);
+	/* build string of newline separated filepaths */
+	for (it = error_list; it != NULL; it = it->next) {
+		char *str;
 
-	dlg = eog_hig_dialog_new (GTK_STOCK_DIALOG_ERROR, _("Could not find files"), path, TRUE);
+		if (n > 9) {
+			/* don't display more than 10 files */
+			detail = g_string_append (detail, "\n...");
+			break;
+		}
+
+		str = gnome_vfs_format_uri_for_display ((char*) it->data);
+
+		if (it != error_list) {
+			detail = g_string_append (detail, "\n");
+		}
+		detail = g_string_append (detail, str);
+
+		g_free (str);
+		n++;
+	}
+	
+	dlg = eog_hig_dialog_new (NULL, GTK_STOCK_DIALOG_ERROR, 
+				  ngettext ("File not found.", "Files not found.", len),
+				  detail->str, TRUE);
 	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
 	
 	gtk_widget_show (dlg);
 	gtk_dialog_run (GTK_DIALOG (dlg));
 	gtk_widget_destroy (dlg);
 
-	g_free (path);
-	g_free (str);
+	g_string_free (detail, TRUE);
 }
 
-static GList*
+static GSList*
 string_array_to_list (const gchar **files)
 {
 	gint i;
-	GList *list = NULL;
+	GSList *list = NULL;
 
 	if (files == NULL) return list;
 
 	for (i = 0; files [i]; i++) {
-		list = g_list_prepend (list, g_strdup (files[i]));
+		list = g_slist_prepend (list, g_strdup (files[i]));
 	}
 
-	return g_list_reverse (list);
+	return g_slist_reverse (list);
 }
 
 static void 
-open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data)
+open_uri_list_cb (EogWindow *window, GSList *uri_list, gpointer data)
 {
 	GList *file_list = NULL;
 	GList *dir_list = NULL;
@@ -426,7 +411,7 @@ open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data)
 	}
 
 	if (quit_program) {
-		bonobo_main_quit ();
+		gtk_main_quit ();
 	}
 }
 
@@ -446,7 +431,7 @@ open_uri_list_cb (EogWindow *window, GList *uri_list, gpointer data)
 static gboolean
 handle_cmdline_args (gpointer data)
 {
-	GList *startup_file_list = NULL;
+	GSList *startup_file_list = NULL;
 	const gchar **startup_files;
 	poptContext ctx;
 
@@ -524,9 +509,6 @@ main (int argc, char **argv)
 	if(gnome_vfs_init () == FALSE)
 		g_error ("Could not initialize GnomeVFS!");
 
-	if (bonobo_ui_init ("Eye of Gnome", VERSION, &argc, argv) == FALSE)
-		g_error ("Could not initialize Bonobo!");
-
 	gnome_window_icon_set_default_from_file (EOG_ICONDIR"/gnome-eog.png");
 
 	client = gnome_master_client ();
@@ -546,7 +528,7 @@ main (int argc, char **argv)
 		g_idle_add (handle_cmdline_args, ctx);
 	}
 
-	bonobo_main ();
+	gtk_main ();
 
 	return 0;
 }
